@@ -44,6 +44,7 @@
 
 #include "error.h"
 #include "common.h"
+#include "connection.h"
 
 #define NETWORK_PEER_INTERFACE "org.bluez.Network"
 
@@ -66,6 +67,7 @@ struct network_conn {
 	uint16_t	id;		/* Role: Service Class Identifier */
 	conn_state	state;
 	int		sk;
+	guint		watch;		/* Disconnect watch */
 	struct network_peer *peer;
 };
 
@@ -124,6 +126,12 @@ static inline DBusMessage *not_connected(DBusMessage *msg)
 						"Device not connected");
 }
 
+static inline DBusMessage *not_permited(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
+						"Operation not permited");
+}
+
 static inline DBusMessage *connection_attempt_failed(DBusMessage *msg, int err)
 {
 	return g_dbus_create_error(msg, ERROR_INTERFACE ".ConnectionAttemptFailed",
@@ -136,13 +144,8 @@ static gboolean bnep_watchdog_cb(GIOChannel *chan, GIOCondition cond,
 	struct network_conn *nc = data;
 
 	if (connection != NULL) {
-		const char *device = nc->dev;
 		gboolean connected = FALSE;
 		const char *property = "";
-		g_dbus_emit_signal(connection, nc->peer->path,
-				NETWORK_PEER_INTERFACE, "Disconnected",
-				DBUS_TYPE_STRING, &device,
-				DBUS_TYPE_INVALID);
 		emit_property_changed(connection, nc->peer->path,
 					NETWORK_PEER_INTERFACE, "Connected",
 					DBUS_TYPE_BOOLEAN, &connected);
@@ -152,6 +155,10 @@ static gboolean bnep_watchdog_cb(GIOChannel *chan, GIOCondition cond,
 		emit_property_changed(connection, nc->peer->path,
 					NETWORK_PEER_INTERFACE, "UUID",
 					DBUS_TYPE_STRING, &property);
+		if (nc->watch) {
+			g_dbus_remove_watch(connection, nc->watch);
+			nc->watch = 0;
+		}
 	}
 
 	info("%s disconnected", nc->dev);
@@ -242,11 +249,6 @@ static gboolean bnep_connect_cb(GIOChannel *chan, GIOCondition cond,
 			DBUS_TYPE_INVALID);
 
 	connected = TRUE;
-	g_dbus_emit_signal(connection, nc->peer->path,
-			NETWORK_PEER_INTERFACE, "Connected",
-			DBUS_TYPE_STRING, &pdev,
-			DBUS_TYPE_STRING, &uuid,
-			DBUS_TYPE_INVALID);
 	emit_property_changed(connection, nc->peer->path,
 				NETWORK_PEER_INTERFACE, "Connected",
 				DBUS_TYPE_BOOLEAN, &connected);
@@ -340,11 +342,28 @@ static void connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 
 failed:
 	nc->state = DISCONNECTED;
+	if (nc->watch) {
+		g_dbus_remove_watch(connection, nc->watch);
+		nc->watch = 0;
+	}
 
 	reply = connection_attempt_failed(nc->msg, -err);
 	g_dbus_send_message(connection, reply);
 }
 
+static void connection_destroy(DBusConnection *conn, void *user_data)
+{
+	struct network_conn *nc = user_data;
+
+	nc->watch = 0;
+
+	if (nc->state == CONNECTED) {
+		bnep_if_down(nc->dev);
+		bnep_kill_connection(&nc->peer->dst);
+	} else
+		close(nc->sk);
+}
+
 /* Connect and initiate BNEP session */
 static DBusMessage *connection_connect(DBusConnection *conn,
 						DBusMessage *msg, void *data)
@@ -369,6 +388,10 @@ static DBusMessage *connection_connect(DBusConnection *conn,
 
 	nc->state = CONNECTING;
 	nc->msg = dbus_message_ref(msg);
+	nc->watch = g_dbus_add_disconnect_watch(conn,
+						dbus_message_get_sender(msg),
+						connection_destroy,
+						nc, NULL);
 
 	err = bt_l2cap_connect(&peer->src, &peer->dst, BNEP_PSM, BNEP_MTU,
 							connect_cb, nc);
@@ -377,6 +400,8 @@ static DBusMessage *connection_connect(DBusConnection *conn,
 		dbus_message_unref(nc->msg);
 		nc->msg = NULL;
 		nc->state = DISCONNECTED;
+		g_dbus_remove_watch(conn, nc->watch);
+		nc->watch = 0;
 		return connection_attempt_failed(msg, -err);
 	}
 
@@ -387,12 +412,18 @@ static DBusMessage *connection_cancel(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
 	struct network_conn *nc = data;
+	const char *owner = dbus_message_get_sender(nc->msg);
+	const char *caller = dbus_message_get_sender(msg);
 
-	if (nc->state == CONNECTED) {
-		bnep_if_down(nc->dev);
-		bnep_kill_connection(&nc->peer->dst);
-	} else
-		close(nc->sk);
+	if (!g_str_equal(owner, caller))
+		return not_permited(msg);
+
+	if (nc->watch) {
+		g_dbus_remove_watch(conn, nc->watch);
+		nc->watch = 0;
+	}
+
+	connection_destroy(conn, data);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -415,26 +446,6 @@ static DBusMessage *connection_disconnect(DBusConnection *conn,
 	return not_connected(msg);
 }
 
-static DBusMessage *is_connected(DBusConnection *conn,
-				DBusMessage *msg, void *data)
-{
-	struct network_peer *peer = data;
-	GSList *l;
-	dbus_bool_t up = FALSE;
-
-	for (l = peer->connections; l; l = l->next) {
-		struct network_conn *nc = l->data;
-
-		if (nc->state != CONNECTED)
-			continue;
-
-		up = TRUE;
-	}
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_BOOLEAN, &up,
-						DBUS_TYPE_INVALID);
-}
-
 static DBusMessage *connection_get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -520,15 +531,11 @@ static GDBusMethodTable connection_methods[] = {
 	{ "Connect",		"s",	"s",	connection_connect,
 						G_DBUS_METHOD_FLAG_ASYNC },
 	{ "Disconnect",		"",	"",	connection_disconnect	},
-	{ "IsConnected",	"",	"b",	is_connected,
-						G_DBUS_METHOD_FLAG_DEPRECATED },
 	{ "GetProperties",	"",	"a{sv}",connection_get_properties },
 	{ }
 };
 
 static GDBusSignalTable connection_signals[] = {
-	{ "Connected",		"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
-	{ "Disconnected",	"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
 	{ "PropertyChanged",	"sv"	},
 	{ }
 };
