@@ -51,11 +51,12 @@
 
 #include "bridge.h"
 #include "common.h"
+#include "server.h"
 
 #define NETWORK_PEER_INTERFACE "org.bluez.NetworkPeer"
 #define NETWORK_HUB_INTERFACE "org.bluez.NetworkHub"
 #define NETWORK_ROUTER_INTERFACE "org.bluez.NetworkRouter"
-#define SETUP_TIMEOUT		1000
+#define SETUP_TIMEOUT		1
 
 /* Pending Authorization */
 struct setup_session {
@@ -380,7 +381,7 @@ static int authorize_connection(struct network_server *ns, const char *address)
 	return ret_val;
 }
 
-static uint16_t inline bnep_setup_chk(uint16_t dst_role, uint16_t src_role)
+static uint16_t bnep_setup_chk(uint16_t dst_role, uint16_t src_role)
 {
 	/* Allowed PAN Profile scenarios */
 	switch (dst_role) {
@@ -555,7 +556,7 @@ static void connect_event(GIOChannel *chan, int err, const bdaddr_t *src,
 	 * the time expires.
 	 */
 	na->to = g_malloc0(sizeof(struct timeout));
-	na->to->id = g_timeout_add(SETUP_TIMEOUT, timeout_cb, na->to);
+	na->to->id = g_timeout_add_seconds(SETUP_TIMEOUT, timeout_cb, na->to);
 	na->to->watch = g_io_add_watch_full(chan, G_PRIORITY_DEFAULT,
 				G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
 				bnep_setup, na, setup_destroy);
@@ -736,15 +737,12 @@ static DBusMessage *get_properties(DBusConnection *conn,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
-	dbus_message_iter_append_dict_entry(&dict, "Name",
-			DBUS_TYPE_STRING, &ns->name);
+	dict_append_entry(&dict, "Name", DBUS_TYPE_STRING, &ns->name);
 
 	uuid = bnep_uuid(ns->id);
-	dbus_message_iter_append_dict_entry(&dict, "Uuid",
-			DBUS_TYPE_STRING, &uuid);
+	dict_append_entry(&dict, "Uuid", DBUS_TYPE_STRING, &uuid);
 
-	dbus_message_iter_append_dict_entry(&dict, "Enabled",
-			DBUS_TYPE_BOOLEAN, &ns->enable);
+	dict_append_entry(&dict, "Enabled", DBUS_TYPE_BOOLEAN, &ns->enable);
 
 	dbus_message_iter_close_container(&iter, &dict);
 
